from datetime import timedelta

from trainingwebsite.certification.utils import string_to_datetime
from trainingwebsite.certification.models import (Cache,
    Account, Hardware, Certificate, CertificateNote)
from trainingwebsite.certification.access import (AccessServer,
    AccountAccess, HardwareAccess, CertificateAccess, CertificateNoteAccess)


class CacheDoesNotExist(Exception):

    pass


class CacheBase(object):
    model_cls = None
    access_cls = None

    def __init__(self, server):
        self.server = server

    def convert(self, **kwargs):
        return kwargs

    def get_cache(self):
        return Cache.objects.get(id=self.model_cls.__name__)

    def create(self, **kwargs):
        kwargs = self.convert(**kwargs)
        return self.model_cls.objects.create(**kwargs)

    def delete(self, id):
        try:
            model = self.model_cls.objects.get(id=id)
            model.delete()
        except self.model_cls.DoesNotExist:
            pass

    def update(self, **kwargs):
        id = kwargs.pop("id")
        try:
            model = self.model_cls.objects.get(id=id)
        except self.model_cls.DoesNotExist:
            raise CacheDoesNotExist

        kwargs = self.convert(**kwargs)
        model.update(**kwargs)

    def sync(self):
        access = self.access_cls(self.server)

        start_datetime = self.last_updated
        end_datetime = string_to_datetime(self.server.get_timestamp())
        if not start_datetime:
            for kwargs in access.get_all():
                self.create(**kwargs)
        else:
            start_datetime -= timedelta(minutes=1)
            for kwargs in access.get_updated(start_datetime, end_datetime):
                try:
                    self.update(**kwargs)
                except CacheDoesNotExist:
                    self.create(**kwargs)

            for id in access.get_deleted(start_datetime, end_datetime):
                self.delete(id)

        self.last_updated = end_datetime

    def _get_last_updated(self):
        cache = self.get_cache()
        return cache.last_updated

    def _set_last_updated(self, last_updated):
        cache = self.get_cache()
        cache.last_updated = last_updated
        cache.save()

    last_updated = property(_get_last_updated, _set_last_updated)


class CacheChildBase(CacheBase):
    parent_cls = None

    @property
    def parent_name(self):
        return self.parent_cls.model_cls.__name__.lower()

    def create(self, **kwargs):
        name = self.parent_name
        kwargs[name] = self.parent_cls.model_cls.objects.get(id=kwargs[name])
        return super(CacheChildBase, self).create(**kwargs)

    def update(self, **kwargs):
        name = self.parent_name
        del kwargs[name]
        super(CacheChildBase, self).update(**kwargs)


class AccountCache(CacheBase):
    model_cls = Account
    access_cls = AccountAccess


class HardwareCache(CacheChildBase):
    model_cls = Hardware
    access_cls = HardwareAccess
    parent_cls = AccountCache


class CertificateCache(CacheChildBase):
    model_cls = Certificate
    access_cls = CertificateAccess
    parent_cls = HardwareCache

    def convert(self, **kwargs):
        kwargs["completed"] = kwargs.pop("date_completed")
        kwargs.pop("notes", None)
        return kwargs


class CertificateNoteCache(CacheChildBase):
    model_cls = CertificateNote
    access_cls = CertificateNoteAccess
    parent_cls = CertificateCache


def synchronize():
    from django.conf import settings

    url = settings.CERTIFICATION_PROXY_URL
    server = AccessServer(url)
    for cache in [AccountCache, HardwareCache, CertificateCache, CertificateNoteCache]:
        cache(server).sync()

def resynchronize():
    Cache.objects.all().delete()
    for cls in CertificateNote, Certificate, Hardware, Account:
        cls.objects.all().delete()
        Cache.objects.create(id=cls.__name__)

    synchronize()
