from datetime import datetime

import unittest

from certification.models import Cache, Account, Hardware, Certificate


class CacheTestCase(unittest.TestCase):

    def setUp(self):
        self.id = "account"
        self.last_updated = datetime.now()

    def testEmpty(self):
        cache = Cache.objects.create(id=self.id)
        self.assertEquals(cache.last_updated, None)

    def testInit(self):
        old_cache = Cache.objects.create(id=self.id,
            last_updated=self.last_updated)
        self.assertEquals(old_cache.last_updated, self.last_updated)

        new_cache = Cache.objects.get(id=self.id)
        self.assertEquals(old_cache, new_cache)

    def testSet(self):
        old_cache = Cache.objects.create(id=self.id)
        old_cache.last_updated = self.last_updated
        self.assertEquals(old_cache.last_updated, self.last_updated)

        new_cache = Cache.objects.get(id=self.id)
        self.assertEquals(old_cache, new_cache)


class AccountTestCase(unittest.TestCase):

    def setUp(self):
        self.id = "0123456789ABCDEFGHI"
        self.name = "foobar"
        self.website = "http://foobar.com"

    def tearDown(self):
        Hardware.objects.all().delete()
        Account.objects.all().delete()

    def testInit(self):
        old_account = Account.objects.create(id=self.id,
            name=self.name, website=self.website)
        self.assertEquals(old_account.name, self.name)
        self.assertEquals(old_account.website, self.website)

        new_account = Account.objects.get(id=self.id)
        self.assertEquals(old_account, new_account)

    def testHardware(self):
        account = Account.objects.create(id=self.id,
            name=self.name, website=self.website)
        self.assertEquals(len(account.hardware), 0)

        hardware = account.create_hardware(id="id",
            name="cid", make="make", models="name",
            architecture="arch", category="cat")
        self.assertEquals(account.hardware[0], hardware)


class HardwareTestCase(unittest.TestCase):

    def setUp(self):
        self.id = "id"
        self.name = "cid"
        self.make = "make"
        self.models = "name"
        self.architecture = "arch"
        self.category = "cat"

        self.account = Account.objects.create(id="id",
            name="name", website="website")

    def tearDown(self):
        Certificate.objects.all().delete()
        Hardware.objects.all().delete()
        Account.objects.all().delete()

    def testInit(self):
        old_hardware = self.account.create_hardware(id=self.id,
            name=self.name, make=self.make, models=self.models,
            category=self.category, architecture=self.architecture)
        self.assertEquals(old_hardware.name, self.name)
        self.assertEquals(old_hardware.make, self.make)
        self.assertEquals(old_hardware.models, self.models)
        self.assertEquals(old_hardware.category, self.category)
        self.assertEquals(old_hardware.architecture, self.architecture)

        new_hardware = Hardware.objects.get(id=self.id)
        self.assertEquals(old_hardware, new_hardware)

    def testCertificates(self):
        hardware = self.account.create_hardware(id=self.id,
            name=self.name, make=self.make, models=self.models,
            category=self.category, architecture=self.architecture)
        self.assertEquals(len(hardware.certificates), 0)

        certificate = hardware.create_certificate(id="id",
            name="name", completed=datetime.now(), release="6.06")
        self.assertEquals(hardware.certificates[0], certificate)


class CertificateTestCase(unittest.TestCase):

    def setUp(self):
        self.id = "id"
        self.name = "name"
        self.completed = datetime.now()
        self.release = "6.06"

        account = Account.objects.create(id="id",
            name="name", website="website")
        self.hardware = account.create_hardware(id="id",
            name="cid", make="make", models="name",
            architecture="arch", category="cat")

    def tearDown(self):
        Certificate.objects.all().delete()
        Hardware.objects.all().delete()
        Account.objects.all().delete()

    def testInit(self):
        old_certificate = self.hardware.create_certificate(id=self.id,
            name=self.name, completed=self.completed, release=self.release)
        self.assertEquals(old_certificate.name, self.name)
        self.assertEquals(old_certificate.completed, self.completed)
        self.assertEquals(old_certificate.release, self.release)

        new_certificate = Certificate.objects.get(id=self.id)
        self.assertEquals(old_certificate, new_certificate)
