from django.template import Context, loader
from django.http import HttpResponse

from trainingwebsite.certification.models import Hardware, Certificate, Entry
from trainingwebsite.certification.utils import convert_url, dict_extract, uniquer


architecture_map = {
    "amd64": "64-bit PC (x86_64)",
    "i386":  "PC (x86)",
    "lpia": "LPIA",
    "sparc": "SPARC"}


def index(request):
    categories = [h["category"] for h in Hardware.objects.all().
        values("category").
        order_by("category").
        distinct()]
    releases = [c["release"] for c in Certificate.objects.all().
        values("release").
        order_by("-release").
        distinct()]
    recents = [vars(e) for e in Entry.objects.all().
        order_by("-completed")]
    recents = uniquer(recents, lambda x: x.get('hardware_name'))[:5]

    context = Context({
        "categories": categories,
        "releases": releases,
        "recents": recents})
    template = loader.get_template("certification/views/index.html")
    return HttpResponse(template.render(context))

def list(request):
    query = dict_extract(request.GET, ["search", "architecture",
        "category", "certification", "release"])
    entries = Entry.objects.complex_filter(**query)
    accounts = _entries_to_accounts(entries)

    context = Context({
        "accounts": accounts})
    template = loader.get_template("certification/views/list.html")
    return HttpResponse(template.render(context))

def advanced(request):
    template = loader.get_template("certification/views/advanced.html")
    return HttpResponse(template.render(context))

def hardware(request, id):
    hardware = Hardware.objects.get(name=id)

    # Convert certificate information
    certificates = hardware.certificates.order_by("-release")
    certificates_context = []
    for certificate in certificates:
        certificate_context = vars(certificate)
        certificate_context["notes"] = [vars(n) for n in certificate.notes]
        certificates_context.append(certificate_context)

    # Convert account information
    account = hardware.account
    account_context = vars(account)
    account_context["website"] = convert_url(account.website)

    # Convert hardware information
    hardware_context = vars(hardware)
    hardware_context["model"] = hardware.model
    hardware_context["models"] = ", ".join(hardware.models.split("\n"))
    hardware_context["status"] = hardware.status
    hardware_context["releases"] = sorted([c.release for c in certificates],
        lambda x, y: cmp(y, x))
    hardware_context["architecture"] = \
        architecture_map[hardware_context["architecture"]]

    context = Context({
        "account": account_context,
        "hardware": hardware_context,
        "certificates": certificates_context})
    template = loader.get_template("certification/views/hardware.html")
    return HttpResponse(template.render(context))

def _entries_to_accounts(entries):
    accounts = {}
    for entry in entries:
        new_entry = vars(entry)

        # Convert fields
        new_entry["model"] = entry.model
        new_entry["status"] = entry.status
        for website in ["account_website", "hardware_website"]:
            new_entry[website] = convert_url(new_entry[website])

        account = {
            "name": new_entry.pop("account_name"),
            "website": new_entry.pop("account_website"),
            "hardware": {}}

        # Make account and hardware entries unique
        account = accounts.setdefault(account["name"], account)
        old_entry = account["hardware"].setdefault(new_entry["hardware_name"],
            new_entry)

        # Set release as a list of names and notes
        if old_entry == new_entry:
            old_entry["website"] = old_entry.pop("hardware_website")
            old_entry["releases"] = [{
                "name": old_entry.pop("release"),
                "notes": old_entry.pop("notes")}]
        else:
            old_entry["releases"].append({
                "name": new_entry["release"],
                "notes": new_entry["notes"]})

    # Set hardware as a list
    for account in accounts.values():
        # Order hardware by certification and then by models
        account["hardware"] = account["hardware"].values()
        account["hardware"] = sorted(account["hardware"],
            lambda x, y: cmp(x["certification"] + x["models"],
                y["certification"] + y["models"]))

        for hardware in account["hardware"]:
            # Set releases as a comma separated list
            hardware["releases"] = sorted(hardware["releases"],
                lambda x, y: cmp(y["name"], x["name"]))
            # Set architecture from map
            hardware["architecture"] = architecture_map[hardware["architecture"]]

    # Set accounts as a list
    return accounts.values()
