from django.db import models
from datetime import datetime
from django.utils.html import strip_tags

# Create your models here.
class Category(models.Model):
	name = models.CharField(maxlength = 80)
	
	class Admin:
		pass
	
	def __str__(self):
		return self.name

class Organization(models.Model):
    name = models.CharField(maxlength = 80)
    shortname = models.CharField(maxlength = 25, help_text = "This is a short version of the organization name that should be all lower case and contain no punctuation or spaces.")
    contact_person = models.TextField(help_text = "This should be in the form of Firstname Lastname <email@address>")
    website = models.URLField(verify_exists = False, help_text = "Please ensure the web address begins with http://")
    
    class Admin:
        pass
    
    def __str__(self):
        return self.name
    
    def save(self):
        self.name = strip_tags(self.name)
        self.shortname = strip_tags(self.shortname)
        self.website = strip_tags(self.website)
        super(Organization, self).save()

class Job(models.Model):
    organization = models.ForeignKey(Organization)
    postid = models.CharField(maxlength = 50, help_text = "This needs to be a short string of numbers or letters. No symbols, spaces or punctuation.")
    name = models.CharField(maxlength = 100)
    postdate = models.DateField(default = datetime.now())
    expires = models.DateField(null = True, blank = True)
    category = models.ForeignKey(Category)
    reportsto = models.TextField(help_text = "To whom will the candidate for this job report?", null = True, blank = True)
    location = models.TextField(help_text = "A one sentence description of the location. No HTML.")
    summary = models.TextField(help_text = "A short description of the job. HTML cannot be used.")
    description = models.TextField(default = """## Key responsibilities: ##

 * item 1
 * item 2
 * item 3

## Required skills and experience: ##

 * item 1
 * item 2
 * item 3
""", help_text = "A long, detailed description of the job. HTML cannot be used but you can use markdown. See the top of this page for details about the markdown syntax")
    howtoapply = models.TextField(default = "http://www.canonical.com/employment/howtoapply", help_text = "Instructions of how to apply or a link to a page that includes instructions (commonly you will link to <a href='http://www.canonical.com/employment/howtoapply'>http://www.canonical.com/employment/howtoapply</a>). Links and e-mail addresses will become clickable. HTML cannot be used.")
    
    class Admin:
         list_display = ('name', 'postid', 'postdate', 'expires','category',)
         list_filter = ('category','organization', 'expires',)
         search_fields = ('name', 'postid','category',)
         ordering = ('-expires', 'category', '-postdate',)
    
    def save(self):
        self.postid = strip_tags(self.postid)
        self.name = strip_tags(self.name)
        self.location = strip_tags(self.location)
        self.summary = strip_tags(self.summary)
        self.description = strip_tags(self.description)
        self.howtoapply = strip_tags(self.howtoapply)
        super(Job, self).save()
    
    def __str__(self):
        return " ".join ((self.name, 
                        self.postdate.strftime("%B %Y"),
                        self.postid))
